# Documents folder location
def my_data_folder():
    
    # Return data folder
    folder = "C:\\Users\\kevin\\Documents\\My Work 02"
    folder += "\\MKS Python\\Data Folder\\"
    return folder


# Notes: 1) the following code performs little error 
# checking because the MKS JSON file or MKS JSON query 
# being processed is expected to be properly prepared 
# and populated; 2) you must define a 'my_data_folder
# function that returns the name of folder in which 
# your JSON file resides.

import requests
import urllib
import json

# Return double or none
def get_double(value):

    # Check for none
    if( value is None ): return None
    
    # Check for errors
    try: return float(value)
    except: return None

# Return double or string
def get_value(value):

    # Check for double
    rslt = get_double(value)
    if( rslt ): return rslt
    
    # Assume string value
    return value

# Return double list or none
def get_double_list(value):

    # Check for none or empty
    if( value is None ): return None
    if( not value ): return []
    
    # Parse each element checking for errors
    try: return list(map(float, value))
     
    # Unsuccessful parse
    except: return []

# MKS datum class
class MKSDatum:
    
    # Default constructor
    def __init__(self):
        
        # Initialize value attributes
        self.value_accuracy = None
        self.value = None
        self.value_units = ""
        
        # Initialize temp attributes
        self.temp_accuracy = None
        self.temperature = None
        self.temp_units = ""
        
        # Initialize pres attributes
        self.pres_accuracy = None
        self.pressure = None
        self.pres_units = ""
        
        # Initialize x attributes
        self.x_accuracies = []
        self.x_values = []
        self.x_units = ""
        
        # Initialize attributes
        self.indent = " " * 12
        self.property = None
        self.reference = ""
        self.comment = ""
         
    # Parse input json dictionary
    def read_json(self, json_dict):
        
        # Composition assignments
        self.x_accuracies = get_double_list(json_dict.get("XAccuracies", []))
        self.x_values = get_double_list(json_dict.get("XValues", []))
        self.x_units = json_dict.get("XUnits", "")
       
        # Value assignments
        self.value_accuracy = get_double(json_dict.get("ValueAccuracy"))
        self.value_units = json_dict.get("ValueUnits", "")
        self.value = get_value(json_dict.get("Value"))
        
        # Temperature assignments
        self.temp_accuracy = get_double(json_dict.get("TempAccuracy"))
        self.temperature = get_double(json_dict.get("Temperature"))
        self.temp_units = json_dict.get("TempUnits", "")
        
        # Pressure assignments
        self.pres_accuracy = get_double(json_dict.get("PresAccuracy"))
        self.pressure = get_double(json_dict.get("Pressure"))
        self.pres_units = json_dict.get("PresUnits")
        
        # General assignments
        self.reference = json_dict.get("Reference", "")
        self.comment = json_dict.get("Comment", "")
        
    # Print attributes
    def output(self):
        
        # Output temperature values
        if( self.temperature ):
            print(self.indent, end="")
            print("Temperature:", end=" ")
            print(self.temperature, end=" ")
            print(self.temp_units)
        
        # Output pressure values
        if( self.pressure ):
            print(self.indent, end="")
            print("Pressure:", end=" ")
            print(self.pressure, end=" ")
            print(self.pres_units)
        
        # Output composition values
        if( self.x_values ):
            print(self.indent, end="")
            print("X Values:", end=" ")
            print(self.x_values, end=" ")
            print(self.x_units)
        
        # Output values
        if( self.value ):
            print(self.indent, end="")
            print("Value:", end=" ")
            print(self.value, end=" ")
            print(self.value_units)
        
        # Extra new line
        print()

        
# MKS property class
class MKSProperty:
    
    # Default constructor
    def __init__(self):
        
        # Initialize attributes
        self.indent = " " * 8
        self.component = 0
        self.entity = None
        self.status = ""
        self.name = ""
        self.data = []
         
    # Parse input json dictionary
    def read_json(self, json_dict):
       
        # Assign attributes
        self.component = int(json_dict["Component"])
        self.status = json_dict["Status"]
        self.name = json_dict["Name"]
        
        # Parse entries into mks datum objects
        for ddict in json_dict["Data"]:
            
            # Create datum instance
            datum = MKSDatum()
            
            # Add datum to list
            self.data.append(datum)
            
            # Parse datum attributes
            datum.read_json(ddict)
            
            # Store property
            datum.property = self
         
    # Print attributes
    def output(self):
            
        # Property attributes
        print(self.indent, end="")
        print(self.name, end=": ")
        print(self.status, end=",  ")
        print(self.component)
            
        # Property data
        for datum in self.data:
            datum.output()
        
        # Extra new line
        if( not self.data ):
            print()


# MKS entity class
class MKSEntity:
    
    # Default constructor
    def __init__(self):
        
        # Initialize attributes
        self.indent = " " * 4
        self.entity_type = ""
        self.identifier = ""
        self.properties = []
        self.request = None
        
    # Parse input json dictionary
    def read_json(self, json_dict):
       
        # Assign attributes
        self.entity_type = json_dict["EntityType"]
        self.identifier = json_dict["Identifier"]
        
        # Parse entries into mks properties
        for pdict in json_dict["Properties"]:
            
            # Create property instance
            prop = MKSProperty()
             
            # Add property to list
            self.properties.append(prop)
           
            # Parse property attributes
            prop.read_json(pdict)
            
            # Store entity
            prop.entity = self
         
    # Print attributes
    def output(self):
            
        # Entity attributes
        print(self.indent, end="")
        print(self.identifier, end=": ")
        print(self.entity_type, end=", ");
        print("Start", end="\n\n");
            
        # Entity properties
        for prop in self.properties: prop.output()
        
        # Entity attributes
        print(self.indent, end="")
        print(self.identifier, end=": ")
        print(self.entity_type, end=", ");
        print("End", end="\n\n\n");


# MKS request class
class MKSRequest:
    
    # Default contructor
    def __init__(self):
           
        # Initialize attributes
        self.application = None
        self.arguments = {}
        self.processor = ""
        self.entities = []
        self.version = ""
        self.date = ""
        self.type = ""

    # Parse input json dictionary
    def read_json(self, json_dict):
       
        # Assign attributes
        self.processor = json_dict["Processor"]
        self.version = json_dict["Version"]
        self.date = json_dict["Date"]
        
        # Parse entries into mks entities
        for edict in json_dict["Entities"]:
            
            # Create entity instance
            entity = MKSEntity()
            
            # Add entity to list
            self.entities.append(entity)
            
            # Parse entity attributes
            entity.read_json(edict)
            
            # Store request
            entity.request = self;
         
    # Print attributes
    def output(self):
            
        # Request attributes
        print("Request processed by " + self.processor)
        print(self.type)
            
        # Request entities
        for entity in self.entities:
            entity.output()


# MKS application class
class MKSApplication:
    
    # Default constructor
    def __init__(self):
        
        # Attributes
        self.request = None
    
    # Parse dictionary input
    def read_json_dict(self, json_dict):
         
        # Create request object if needed
        if( self.request is None): self.request = MKSRequest()
        
        # Parse input dictionary
        self.request.read_json(json_dict)
        
        # Store application
        self.request.application = self
       
    # Read input json string
    def read_json(self, json_string):
                
        # Parse json string into dictionary
        dict = json.loads(json_string)
        
        # Parse request object
        self.read_json_dict(dict)            
                
    # Read input json file
    def read_json_file(self, filename):
                
        # Parse json file into dictionary
        with open(filename) as json_data:
            dict = json.load(json_data)
                  
        # Parse request object
        self.read_json_dict(dict)            
 
    # Parse mks api response
    def parse_mks_api(self, query):
        
        # Prepare request
        api_url = "https://mkswebapi.com/process?request="
        query = api_url + urllib.parse.quote_plus(query)
        response = requests.get(query)
                
        # Otherwise process request
        self.read_json_dict(response.json())
         
    # Print attributes
    def output(self):

        # Output request
        self.request.output()
        

# Example F01: json from file
def example_f01_mks_file_usage():

    # Create mks application
    app = MKSApplication();
    
    # File location and name. Note that a my_data_folder
    # function must be defined specific to your computer
    # environment. The function must return the folder
    # in which your json data files are located.
    path = my_data_folder() + "MKSProperties.json"
    app.read_json_file(path)
   
    # Output results
    app.output()


# Imports for example F02         
import matplotlib.pyplot as plt

# Example F02: json from file
def example_f02_mks_file_usage():

    # Initializations
    tb = []
    
    # Create the mks application
    app = MKSApplication();
    
    # File location and name. Note that a my_data_folder
    # function must be defined specific to your computer
    # environment. The function must return the folder
    # in which your json data files are located.
    path = my_data_folder() + "MKSBoilingPoints.json"
    app.read_json_file(path)
    
    # Compile all points from json
    for entity in app.request.entities:

        # Check for boiling point data        
        if( entity.properties and entity.properties[0].data ):
            
            # Retrieve and add boiling point datum
            tb.append(entity.properties[0].data[0].value)

    # Prepare histogram
    plt.title('Histogram of Normal Boiling Points')
    plt.xlabel('Boiling Point, K')
    plt.axis([0, 720, 0, 200])
    plt.ylabel('Count')
    plt.grid(True)
    
    # Plot histogram
    plt.hist(tb, bins=11, edgecolor="black",
        rwidth=0.8)
    
    # Show histogram
    plt.show()
  

# Example F03: json from file
def example_f03_mks_file_usage():

    # Create mks application
    app = MKSApplication();
    
    # File location and name. Note that a my_data_folder
    # function must be defined specific to your computer
    # environment. The function must return the folder
    # in which your json data files are located.
    path = my_data_folder() + "MKSSynonyms.json"
    app.read_json_file(path)
   
    # Output each entity name
    for entity in app.request.entities:
        
        # Print entity name
        print(entity.identifier)
        
        # Check for synonyms
        if( entity.properties ):
            
            # Print synonyms
            for datum in entity.properties[0].data:
            
                # Print datum value
                print("  " + datum.value)
            
        # Print extra line
        print()
       

# Example W01: json from website
def example_w01_mks_api_usage():

    # Create mks application
    app = MKSApplication();
    
    # Prepare api query   
    query = '''{
      "Type": "Get Values",
      "Entities": [
        {
          "Identifier": "1,2-Dichlorobenzene",
          "EntityType": "Chemical",
          "Properties": [
            {
              "Name": "Boiling Point",
              "Status": "Active",
              "Component": "0"
            },
            {
              "Name": "Boiling Point",
              "Status": "New Estimate",
              "Component": "0"
            }
          ]
        }
      ]
    }'''

    # Query api
    app.parse_mks_api(query)
    app.output()
                

# Example W02: json from website
def example_w02_mks_api_usage():

    # Create mks application
    app = MKSApplication();
    
    # Prepare api query   
    query = '''{
      "Type": "Get Values",
      "Entities": [
        {
          "Identifier": "#Virtual Mixture",
          "EntityType": "Mixture",
          "Properties": [
            {
              "Name": "Component",
              "Status": "Active",
              "Component": "0",
              "Data": [
                {
                  "Value": "Methanol"
                },
                {
                  "Value": "Ethanol"
                }
              ]
            },
            {
              "Name": "Activity Coefficient, VLE - f(T,P,X)",
              "Status": "New Estimate",
              "Component": "0",
              "Data": [
                {
                  "TempUnits": "C",
                  "Temperature": "25.0",
                  "PresUnits": "kPa",
                  "Pressure": "101.3",
                  "XUnits": "wt %",
                  "XValues": ["25", "75"]
                },
                {
                  "TempUnits": "C",
                  "Temperature": "25.0",
                  "PresUnits": "kPa",
                  "Pressure": "101.3",
                  "XUnits": "wt %",
                  "XValues": ["50", "50"]
                },
                {
                  "TempUnits": "C",
                  "Temperature": "25.0",
                  "PresUnits": "kPa",
                  "Pressure": "101.3",
                  "XUnits": "wt %",
                  "XValues": ["75", "25"]
                }
              ]
            }
          ]
        }
      ]
    }'''

    # Query api
    app.parse_mks_api(query)
    app.output()
         

# Example W03: json from website
def example_w03_mks_api_usage():

    # Create mks application
    app = MKSApplication();
    
    # Prepare api query   
    query = '''{
      "Type": "Get Values",
      "Entities": [
        {
          "Identifier": "#Virtual Chemical",
          "EntityType": "Chemical",
          "Properties": [
            {
              "Name": "Boiling Point",
              "Status": "New Estimate",
              "Component": "0"
            },
            {
              "Name": "Critical Temperature",
              "Status": "New Estimate",
              "Component": "0"
            },
            {
              "Name": "Flash Point, Closed Cup",
              "Status": "New Estimate",
              "Component": "0"
            },
            {
              "Name": "Smiles String",
              "Status": "Active",
              "Component": "0",
              "Data": [
                {
                  "Value": "CC(SSC(C)C)C"
                }
              ]
            }
          ]
        }
      ]
    }'''

    # Query api
    app.parse_mks_api(query)
    app.output()
     

# Examples - uncomment to run
# example_f01_mks_file_usage()
# example_f02_mks_file_usage()
# example_f03_mks_file_usage()
# example_w01_mks_api_usage()
# example_w02_mks_api_usage()
example_w03_mks_api_usage()
